@@ -24,12 +24,12 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <string.h>
 #include <stdlib.h>
+#include <atk/atkobject.h>
+#include <atk/atkrelationset.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkvbox.h>
@@ -162,6 +162,10 @@ gnome_font_selection_init (GnomeFontSelection * fontsel)
 	GtkTreeSelection *selection;
 	GtkTreeViewColumn *col;
 	GtkCellRenderer *rend;
+	AtkRelationSet *relation_set;
+	AtkRelation *relation;
+	AtkObject *relation_targets[1];
+	AtkObject *atko;
 
 	gtk_box_set_homogeneous ((GtkBox *) fontsel, TRUE);
 	gtk_box_set_spacing ((GtkBox *) fontsel, 4);
@@ -205,6 +209,10 @@ gnome_font_selection_init (GnomeFontSelection * fontsel)
 	fontsel->family = tv;
 	fontsel->selectedfamily = NULL;
 
+	atko = gtk_widget_get_accessible (tv);
+	atk_object_set_name (atko, _("Font family"));
+	atk_object_set_description (atko, _("The list of font families available"));
+
 	/* Fontbox */
 
 	vb = gtk_vbox_new (FALSE, 4);
@@ -258,6 +266,10 @@ gnome_font_selection_init (GnomeFontSelection * fontsel)
 	fontsel->style = tv;
 	fontsel->selectedface = NULL;
 
+	atko = gtk_widget_get_accessible (tv);
+	atk_object_set_name (atko, _("Font style"));
+	atk_object_set_description (atko, _("The list of styles available for the selected font family"));
+
 	/* Sizebox */
 
 	hb = gtk_hbox_new (FALSE, 4);
@@ -289,9 +301,21 @@ gnome_font_selection_init (GnomeFontSelection * fontsel)
 	gtk_entry_set_text ((GtkEntry *) ((GtkCombo *) c)->entry, "12");
 	fontsel->selectedsize = 12.0;
 
-	l = gtk_label_new (_("Font size:"));
+	l = gtk_label_new_with_mnemonic (_("Font _size:"));
 	gtk_widget_show (l);
 	gtk_box_pack_end ((GtkBox *) hb, l, FALSE, FALSE, 0);
+	gtk_label_set_mnemonic_widget ((GtkLabel *) l,
+				       ((GtkCombo *) c)->entry);
+
+	/* Add a LABELLED_BY relation from the combo to the label. */
+	atko = gtk_widget_get_accessible (c);
+	relation_set = atk_object_ref_relation_set (atko);
+	relation_targets[0] = gtk_widget_get_accessible (l);
+	relation = atk_relation_new (relation_targets, 1,
+				     ATK_RELATION_LABELLED_BY);
+	atk_relation_set_add (relation_set, relation);
+	g_object_unref (G_OBJECT (relation));
+	g_object_unref (G_OBJECT (relation_set));
 }
 
 static void
@@ -877,6 +901,7 @@ static void
 gnome_font_dialog_init (GnomeFontDialog *fontseldiag)
 {
 	GnomeFont * font;
+	AtkObject *atko;
 
 	gtk_window_set_default_size (GTK_WINDOW (fontseldiag), 500, 300);
 	
@@ -896,6 +921,10 @@ gnome_font_dialog_init (GnomeFontDialog *fontseldiag)
 	gtk_widget_show (fontseldiag->preview);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (fontseldiag)->vbox), fontseldiag->preview, TRUE, TRUE, 0);
 
+	atko = gtk_widget_get_accessible (fontseldiag->preview);
+	atk_object_set_name (atko, _("Font Preview"));
+	atk_object_set_description (atko, _("Displays some example text in the selected font"));
+
 	font = gnome_font_selection_get_font ((GnomeFontSelection *) fontseldiag->fontsel);
 	gnome_font_preview_set_font ((GnomeFontPreview *) fontseldiag->preview, font);
 
