@@ -25,13 +25,11 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
-
-#include "gnome-print-i18n.h"
 
 #include <time.h>
+#include <atk/atkobject.h>
+#include <atk/atkrelationset.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkdialog.h>
@@ -45,15 +43,22 @@
 #include <gtk/gtkentry.h>
 #include <gtk/gtkspinbutton.h>
 #include <gtk/gtkstock.h>
+#include <gtk/gtkbbox.h>
 
 #include <libgnomeprint/gnome-print-config.h>
 
+#include "gnome-print-i18n.h"
 #include "gnome-printer-dialog.h"
 #include "gnome-print-paper-selector.h"
 #include "gnome-print-copies.h"
 #include "gnome-print-dialog.h"
 
-#define PAD 4
+#define PAD 6
+
+enum {
+	ARG_0,
+	ARG_PRINT_CONFIG
+};
 
 struct _GnomePrintDialog {
 	GtkDialog dialog;
@@ -110,6 +115,30 @@ gnome_print_dialog_destroy (GtkObject *object)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
+static void
+gnome_print_dialog_set_property (GObject      *object,
+				 guint         prop_id,
+				 GValue const *value,
+				 GParamSpec   *pspec)
+{
+	gpointer new_config;
+	GnomePrintDialog *gpd = GNOME_PRINT_DIALOG (object);
+
+	switch (prop_id) {
+	case ARG_PRINT_CONFIG:
+		new_config = g_value_get_pointer (value);
+		if (new_config) {
+			if (gpd->config)
+				gnome_print_config_unref (gpd->config);
+			gpd->config = g_value_get_pointer (value);
+			gnome_print_config_ref (gpd->config);
+		}
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+	}
+}
+
 static void
 gnome_print_dialog_class_init (GnomePrintDialogClass *class)
 {
@@ -120,48 +149,21 @@ gnome_print_dialog_class_init (GnomePrintDialogClass *class)
 	parent_class = gtk_type_class (GTK_TYPE_DIALOG);
 
 	object_class->destroy = gnome_print_dialog_destroy;
+
+	G_OBJECT_CLASS (class)->set_property = gnome_print_dialog_set_property;
+	g_object_class_install_property (G_OBJECT_CLASS (class),
+		ARG_PRINT_CONFIG,
+		g_param_spec_pointer ("print_config",
+			"Print Config",
+			"Printing Configuration to be used",
+			G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE));
 }
 
 static void
 gnome_print_dialog_init (GnomePrintDialog *gpd)
 {
-	gpd->config = gnome_print_config_default ();
-
-	if (gpd->config) {
-		GtkWidget *hb, *vb, *p, *l;
-		gpd->notebook = gtk_notebook_new ();
-		gtk_widget_show (gpd->notebook);
-		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (gpd)->vbox), gpd->notebook, TRUE, TRUE, 0);
-		/* Job page */
-		gpd->job = gpd_create_job_page (gpd);
-		gtk_widget_show (gpd->job);
-		l = gtk_label_new (_("Job"));
-		gtk_widget_show (l);
-		gtk_notebook_append_page (GTK_NOTEBOOK (gpd->notebook), gpd->job, l);
-		/* Printers page */
-		hb = gtk_hbox_new (FALSE, 0);
-		gtk_widget_show (hb);
-		vb = gtk_vbox_new (FALSE, 0);
-		gtk_widget_show (vb);
-		gtk_box_pack_start (GTK_BOX (hb), vb, FALSE, FALSE, 0);
-		gpd->printer = gnome_printer_selection_new (gpd->config);
-		gtk_widget_show (gpd->printer);
-		gtk_box_pack_start (GTK_BOX (vb), gpd->printer, FALSE, FALSE, 0);
-		l = gtk_label_new (_("Printer"));
-		gtk_widget_show (l);
-		gtk_notebook_append_page (GTK_NOTEBOOK (gpd->notebook), hb, l);
-		/* Paper page */
-		p = gnome_paper_selector_new (gpd->config);
-		gtk_widget_show (p);
-		l = gtk_label_new (_("Paper"));
-		gtk_widget_show (l);
-		gtk_notebook_append_page (GTK_NOTEBOOK (gpd->notebook), p, l);
-	} else {
-		GtkWidget *label;
-		label = gtk_label_new (_("Error in loading printer configuration"));
-		gtk_widget_show (label);
-		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (gpd)->vbox), label, TRUE, TRUE, 0);
-	}
+	if (gpd->config == NULL)
+		gpd->config = gnome_print_config_default ();
 }
 
 static void
@@ -182,17 +184,18 @@ gpd_create_job_page (GnomePrintDialog *gpd)
 
 	vb = gtk_vbox_new (FALSE, PAD);
 	gtk_widget_show (vb);
-	gtk_box_pack_start (GTK_BOX (hb), vb, FALSE, FALSE, PAD);
 
-	f = gtk_frame_new (_("Print Range"));
+	gtk_box_pack_start (GTK_BOX (hb), vb, FALSE, FALSE, 0);
+
+	f = gtk_frame_new (_("Print range"));
 	gtk_widget_hide (f);
-	gtk_box_pack_start (GTK_BOX (vb), f, FALSE, FALSE, PAD);
+	gtk_box_pack_start (GTK_BOX (vb), f, FALSE, FALSE, 0);
 	gtk_object_set_data (GTK_OBJECT (hb), "range", f);
 
 	c = gnome_print_copies_selection_new ();
 	if (gpd) gtk_signal_connect (GTK_OBJECT (c), "copies_set", GTK_SIGNAL_FUNC (gpd_copies_set), gpd);
 	gtk_widget_hide (c);
-	gtk_box_pack_start (GTK_BOX (vb), c, FALSE, FALSE, PAD);
+	gtk_box_pack_start (GTK_BOX (vb), c, FALSE, FALSE, 0);
 	gtk_object_set_data (GTK_OBJECT (hb), "copies", c);
 
 	return hb;
@@ -211,39 +214,44 @@ gpd_create_range (gint flags, GtkWidget *range, const guchar *clabel, const guch
 	row = 0;
 
 	if (flags & GNOME_PRINT_RANGE_CURRENT) {
-		rb = gtk_radio_button_new_with_label (group, clabel);
+		rb = gtk_radio_button_new_with_mnemonic (group, clabel);
 		gtk_object_set_data (GTK_OBJECT (t), "current", rb);
 		gtk_widget_show (rb);
-		gtk_table_attach (GTK_TABLE (t), rb, 0, 1, row, row + 1, GTK_FILL, GTK_FILL, 0, 0);
+		gtk_table_attach (GTK_TABLE (t), rb, 0, 1, row, row + 1, GTK_FILL | GTK_EXPAND, 
+				  GTK_FILL, 0, 0);
 		group = gtk_radio_button_group (GTK_RADIO_BUTTON (rb));
 		row += 1;
 	}
 
 	if (flags & GNOME_PRINT_RANGE_ALL) {
-		rb = gtk_radio_button_new_with_label(group, _("All"));
+		rb = gtk_radio_button_new_with_mnemonic(group, _("_All"));
 		gtk_object_set_data (GTK_OBJECT (t), "all", rb);
 		gtk_widget_show (rb);
-		gtk_table_attach (GTK_TABLE (t), rb, 0, 1, row, row + 1, GTK_FILL, GTK_FILL, 0, 0);
+		gtk_table_attach (GTK_TABLE (t), rb, 0, 1, row, row + 1, GTK_FILL | GTK_EXPAND, 
+				  GTK_FILL, 0, 0);
 		group = gtk_radio_button_group (GTK_RADIO_BUTTON (rb));
 		row += 1;
 	}
 
 	if (flags & GNOME_PRINT_RANGE_RANGE) {
-		rb = gtk_radio_button_new_with_label (group, rlabel);
+		rb = gtk_radio_button_new_with_mnemonic (group, rlabel);
 		gtk_object_set_data (GTK_OBJECT (t), "range", rb);
 		gtk_widget_show (rb);
-		gtk_table_attach (GTK_TABLE (t), rb, 0, 1, row, row + 1, GTK_FILL, GTK_SHRINK, 0, 0);
+		gtk_table_attach (GTK_TABLE (t), rb, 0, 1, row, row + 1, GTK_FILL | GTK_EXPAND, 
+				  GTK_FILL, 0, 0);
+		gtk_object_set_data (GTK_OBJECT (t), "range-widget", range);
 		gtk_table_attach (GTK_TABLE (t), range, 1, 2, row, row + 1, GTK_FILL, 0, 0, 0);
 		group = gtk_radio_button_group (GTK_RADIO_BUTTON (rb));
 		row += 1;
 	}
 
 	if ((flags & GNOME_PRINT_RANGE_SELECTION) || (flags & GNOME_PRINT_RANGE_SELECTION_UNSENSITIVE)) {
-		rb = gtk_radio_button_new_with_label (group, _("Selection"));
+		rb = gtk_radio_button_new_with_mnemonic (group, _("_Selection"));
 		gtk_object_set_data (GTK_OBJECT (t), "selection", rb);
 		gtk_widget_show (rb);
 		gtk_widget_set_sensitive (rb, !(flags & GNOME_PRINT_RANGE_SELECTION_UNSENSITIVE));
-		gtk_table_attach (GTK_TABLE (t), rb, 0, 1, row, row + 1, GTK_FILL, GTK_FILL, 0, 0);
+		gtk_table_attach (GTK_TABLE (t), rb, 0, 1, row, row + 1, GTK_FILL | GTK_EXPAND, 
+				  GTK_FILL, 0, 0);
 		group = gtk_radio_button_group (GTK_RADIO_BUTTON (rb));
 		row += 1;
 	}
@@ -274,7 +282,8 @@ gnome_print_dialog_new (const guchar *title, gint flags)
 
 	w = gtk_type_new (GNOME_TYPE_PRINT_DIALOG);
 
-	if (w) gnome_print_dialog_construct (GNOME_PRINT_DIALOG (w), title, flags);
+	if (w)
+		gnome_print_dialog_construct (GNOME_PRINT_DIALOG (w), title, flags);
 
 	return w;
 }
@@ -291,28 +300,92 @@ gnome_print_dialog_new (const guchar *title, gint flags)
 void
 gnome_print_dialog_construct (GnomePrintDialog *gpd, const guchar *title, gint flags)
 {
+	GtkWidget *pp_button; 
+
 	g_return_if_fail (gpd != NULL);
 	g_return_if_fail (GNOME_IS_PRINT_DIALOG (gpd));
 
+	if (gpd->config) {
+		GtkWidget *hb, *p, *l;
+		gpd->notebook = gtk_notebook_new ();
+		gtk_container_set_border_width (GTK_CONTAINER (gpd->notebook), 4);
+		gtk_widget_show (gpd->notebook);
+	
+		gtk_container_add (GTK_CONTAINER (GTK_DIALOG (gpd)->vbox), gpd->notebook);
+		
+		/* Add the job page */
+		gpd->job = gpd_create_job_page (gpd);
+		gtk_container_set_border_width (GTK_CONTAINER (gpd->job), 4);
+		if (flags)
+			gtk_widget_show (gpd->job);
+		l = gtk_label_new_with_mnemonic (_("_Job"));
+		gtk_widget_show (l);
+		
+		gtk_notebook_append_page (GTK_NOTEBOOK (gpd->notebook), gpd->job, l);
+		
+		/* Add the printers page */
+		hb = gtk_hbox_new (FALSE, 0);
+		gtk_widget_show (hb);
+		
+		gpd->printer = gnome_printer_selection_new (gpd->config);
+		gtk_container_set_border_width (GTK_CONTAINER (hb), 4);
+		gtk_widget_show (gpd->printer);
+		
+		gtk_box_pack_start (GTK_BOX (hb), gpd->printer, FALSE, FALSE, 0);
+		
+		l = gtk_label_new_with_mnemonic (_("P_rinter"));
+		gtk_widget_show (l);
+		
+		gtk_notebook_append_page (GTK_NOTEBOOK (gpd->notebook), hb, l);
+		
+		/* Add a paper page */
+		p = gnome_paper_selector_new (gpd->config);
+		gtk_container_set_border_width (GTK_CONTAINER (p), 4);
+		gtk_widget_show (p);
+		
+		l = gtk_label_new_with_mnemonic (_("Pap_er"));
+		gtk_widget_show (l);
+		
+		gtk_notebook_append_page (GTK_NOTEBOOK (gpd->notebook), p, l);
+	} 
+	else {
+		GtkWidget *label;
+		label = gtk_label_new (_("Error in loading printer configuration"));
+		gtk_widget_show (label);
+		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (gpd)->vbox), label, TRUE, TRUE, 0);
+	}
+
 	gtk_window_set_title (GTK_WINDOW (gpd), (title) ? title : (const guchar *) _("Gnome Print Dialog"));
 
 	gtk_dialog_add_buttons (GTK_DIALOG (gpd),
-				_("Print"), GNOME_PRINT_DIALOG_RESPONSE_PRINT,
-				_("Preview"), GNOME_PRINT_DIALOG_RESPONSE_PREVIEW,
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				GTK_STOCK_PRINT, GNOME_PRINT_DIALOG_RESPONSE_PRINT,
 				NULL);
 
+	pp_button = gtk_dialog_add_button (GTK_DIALOG (gpd),
+					   GTK_STOCK_PRINT_PREVIEW, 
+					   GNOME_PRINT_DIALOG_RESPONSE_PREVIEW);
+
+	gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (GTK_DIALOG (gpd)->action_area), 
+					    pp_button, 
+					    TRUE);
+
+	gtk_dialog_set_default_response (GTK_DIALOG (gpd),
+					 GNOME_PRINT_DIALOG_RESPONSE_PRINT);
+
 	/* Construct job options, if needed */
 	if (flags & GNOME_PRINT_DIALOG_RANGE) {
 		GtkWidget *r;
 		r = gtk_object_get_data (GTK_OBJECT (gpd->job), "range");
-		if (r) gtk_widget_show (r);
+		if (r)
+			gtk_widget_show (r);
 	}
 
 	if (flags & GNOME_PRINT_DIALOG_COPIES) {
 		GtkWidget *c;
 		c = gtk_object_get_data (GTK_OBJECT (gpd->job), "copies");
-		if (c) gtk_widget_show (c);
+		if (c)
+			gtk_widget_show (c);
 	}
 }
 
@@ -336,10 +409,12 @@ gnome_print_dialog_construct_range_custom (GnomePrintDialog *gpd, GtkWidget *cus
 
 	f = gtk_object_get_data (GTK_OBJECT (gpd->job), "range");
 	g_return_if_fail (f != NULL);
-	r = gtk_object_get_data (GTK_OBJECT (r), "range");
-	if (r) gtk_container_remove (GTK_CONTAINER (f), r);
+	r = gtk_object_get_data (GTK_OBJECT (f), "range");
+	if (r)
+		gtk_container_remove (GTK_CONTAINER (f), r);
 
 	gtk_widget_show (custom);
+	gtk_widget_show (gpd->job);
 	gtk_container_add (GTK_CONTAINER (f), custom);
 	gtk_object_set_data (GTK_OBJECT (f), "range", custom);
 }
@@ -378,12 +453,14 @@ gnome_print_dialog_construct_range_any (GnomePrintDialog *gpd, gint flags, GtkWi
 	f = gtk_object_get_data (GTK_OBJECT (gpd->job), "range");
 	g_return_if_fail (f != NULL);
 	r = gtk_object_get_data (GTK_OBJECT (f), "range");
-	if (r) gtk_container_remove (GTK_CONTAINER (f), r);
+	if (r)
+		gtk_container_remove (GTK_CONTAINER (f), r);
 
 	r = gpd_create_range (flags, range_widget, currentlabel, rangelabel);
 
 	if (r) {
 		gtk_widget_show (r);
+		gtk_widget_show (gpd->job);
 		gtk_container_add (GTK_CONTAINER (f), r);
 	}
 
@@ -412,11 +489,15 @@ gnome_print_dialog_construct_range_page (GnomePrintDialog *gpd, gint flags, gint
 	if (flags & GNOME_PRINT_RANGE_RANGE) {
 		GtkWidget *l, *sb;
 		GtkObject *a;
+		AtkRelationSet *relation_set;
+		AtkRelation *relation;
+		AtkObject *relation_targets[1];
+		AtkObject *atko;
 
 		hbox = gtk_hbox_new (FALSE, 3);
 		gtk_widget_show (hbox);
 
-		l = gtk_label_new (_("From:"));
+		l = gtk_label_new_with_mnemonic (_("_From:"));
 		gtk_widget_show (l);
 		gtk_box_pack_start (GTK_BOX (hbox), l, FALSE, FALSE, 0);
 
@@ -425,16 +506,44 @@ gnome_print_dialog_construct_range_page (GnomePrintDialog *gpd, gint flags, gint
 		sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 1, 0.0);
 		gtk_widget_show (sb);
 		gtk_box_pack_start (GTK_BOX (hbox), sb, FALSE, FALSE, 0);
+		gtk_label_set_mnemonic_widget ((GtkLabel *) l, sb);
 
-		l = gtk_label_new (_("To:"));
+		/* Add a LABELLED_BY relation from the spinbutton to the label.
+		 */
+		atko = gtk_widget_get_accessible (sb);
+		atk_object_set_description (atko, _("Sets the start of the range of pages to be printed"));
+
+		relation_set = atk_object_ref_relation_set (atko);
+		relation_targets[0] = gtk_widget_get_accessible (l);
+		relation = atk_relation_new (relation_targets, 1,
+					     ATK_RELATION_LABELLED_BY);
+		atk_relation_set_add (relation_set, relation);
+		g_object_unref (G_OBJECT (relation));
+		g_object_unref (G_OBJECT (relation_set));
+
+		l = gtk_label_new_with_mnemonic (_("_To:"));
 		gtk_widget_show (l);
 		gtk_box_pack_start (GTK_BOX (hbox), l, FALSE, FALSE, 0);
 
-		a = gtk_adjustment_new (start, start, end, 1, 10, 10);
+		a = gtk_adjustment_new (end, start, end, 1, 10, 10);
 		gtk_object_set_data (GTK_OBJECT (hbox), "to", a);
 		sb = gtk_spin_button_new (GTK_ADJUSTMENT (a), 1, 0.0);
 		gtk_widget_show (sb);
 		gtk_box_pack_start (GTK_BOX (hbox), sb, FALSE, FALSE, 0);
+		gtk_label_set_mnemonic_widget ((GtkLabel *) l, sb);
+
+		/* Add a LABELLED_BY relation from the spinbutton to the label.
+		 */
+		atko = gtk_widget_get_accessible (sb);
+		atk_object_set_description (atko, _("Sets the end of the range of pages to be printed"));
+
+		relation_set = atk_object_ref_relation_set (atko);
+		relation_targets[0] = gtk_widget_get_accessible (l);
+		relation = atk_relation_new (relation_targets, 1,
+					     ATK_RELATION_LABELLED_BY);
+		atk_relation_set_add (relation_set, relation);
+		g_object_unref (G_OBJECT (relation));
+		g_object_unref (G_OBJECT (relation_set));
 	}
 
 	gnome_print_dialog_construct_range_any (gpd, flags, hbox, currentlabel, rangelabel);
@@ -468,13 +577,20 @@ gnome_print_dialog_get_range (GnomePrintDialog *gpd)
 	g_return_val_if_fail (r != NULL, 0);
 
 	b = gtk_object_get_data (GTK_OBJECT (r), "current");
-	if (b && GTK_IS_TOGGLE_BUTTON (b) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) return GNOME_PRINT_RANGE_CURRENT;
+	if (b && GTK_IS_TOGGLE_BUTTON (b) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) 
+		return GNOME_PRINT_RANGE_CURRENT;
+	
 	b = gtk_object_get_data (GTK_OBJECT (r), "all");
-	if (b && GTK_IS_TOGGLE_BUTTON (b) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) return GNOME_PRINT_RANGE_ALL;
+	if (b && GTK_IS_TOGGLE_BUTTON (b) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) 
+		return GNOME_PRINT_RANGE_ALL;
+
 	b = gtk_object_get_data (GTK_OBJECT (r), "range");
-	if (b && GTK_IS_TOGGLE_BUTTON (b) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) return GNOME_PRINT_RANGE_RANGE;
+	if (b && GTK_IS_TOGGLE_BUTTON (b) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) 
+		return GNOME_PRINT_RANGE_RANGE;
+	
 	b = gtk_object_get_data (GTK_OBJECT (r), "selection");
-	if (b && GTK_IS_TOGGLE_BUTTON (b) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) return GNOME_PRINT_RANGE_SELECTION;
+	if (b && GTK_IS_TOGGLE_BUTTON (b) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (b))) 
+		return GNOME_PRINT_RANGE_SELECTION;
 
 	return 0;
 }
@@ -502,17 +618,21 @@ gnome_print_dialog_get_range_page (GnomePrintDialog *gpd, gint *start, gint *end
 	mask = gnome_print_dialog_get_range (gpd);
 
 	if (mask & GNOME_PRINT_RANGE_RANGE) {
-		GtkObject *f, *r, *a;
+		GtkObject *f, *r, *w, *a;
 		f = gtk_object_get_data (GTK_OBJECT (gpd->job), "range");
 		g_return_val_if_fail (f != NULL, 0);
 		r = gtk_object_get_data (GTK_OBJECT (f), "range");
 		g_return_val_if_fail (r != NULL, 0);
-		a = gtk_object_get_data (GTK_OBJECT (r), "from");
+		w = gtk_object_get_data (GTK_OBJECT (r), "range-widget");
+		g_return_val_if_fail (w != NULL, 0);
+		a = gtk_object_get_data (GTK_OBJECT (w), "from");
 		g_return_val_if_fail (a && GTK_IS_ADJUSTMENT (a), 0);
-		if (start) *start = (gint) gtk_adjustment_get_value (GTK_ADJUSTMENT (a));
-		a = gtk_object_get_data (GTK_OBJECT (r), "to");
+		if (start)
+			*start = (gint) gtk_adjustment_get_value (GTK_ADJUSTMENT (a));
+		a = gtk_object_get_data (GTK_OBJECT (w), "to");
 		g_return_val_if_fail (a && GTK_IS_ADJUSTMENT (a), 0);
-		if (end) *end = (gint) gtk_adjustment_get_value (GTK_ADJUSTMENT (a));
+		if (end)
+			*end = (gint) gtk_adjustment_get_value (GTK_ADJUSTMENT (a));
 	}
 
 	return mask;
@@ -542,8 +662,10 @@ gnome_print_dialog_get_copies (GnomePrintDialog *gpd, gint *copies, gint *collat
 
 		c = gtk_object_get_data (GTK_OBJECT (gpd->job), "copies");
 		if (c && GNOME_IS_PRINT_COPIES_SELECTION (c)) {
-			if (copies) *copies = gnome_print_copies_selection_get_copies (c);
-			if (collate) *collate = gnome_print_copies_selection_get_collate (c);
+			if (copies)
+				*copies = gnome_print_copies_selection_get_copies (c);
+			if (collate)
+				*collate = gnome_print_copies_selection_get_collate (c);
 		}
 	}
 }
