@@ -38,32 +38,34 @@ typedef struct _GnomePrintDialogClass GnomePrintDialogClass;
 #include <sys/types.h>
 #include <gtk/gtkwidget.h>
 
-/* used to track what type of range selector has been setup */
+/*
+ * Used to track what type of range selector has been setup
+ *
+ * This is used internally and has little use for outside world
+ */
+
 typedef enum {
 	GNOME_PRINT_RANGETYPE_NONE,
 	GNOME_PRINT_RANGETYPE_CUSTOM,
-	GNOME_PRINT_RANGETYPE_PAGES,
-#if 0
-	GNOME_PRINT_RANGETYPE_DATES
-#endif
+	GNOME_PRINT_RANGETYPE_PAGES
 } GnomePrintRangeType;
 
-/* what options on the range selector do we want? */
+/*
+ * Flags for creation of range widget
+ */
+
 enum GnomePrintDialogRangeFlags {
 	GNOME_PRINT_RANGE_CURRENT = (1 << 0),
 	GNOME_PRINT_RANGE_ALL = (1 << 1),
 	GNOME_PRINT_RANGE_RANGE = (1 << 2),
 	GNOME_PRINT_RANGE_SELECTION = (1 << 3),
 	GNOME_PRINT_RANGE_SELECTION_UNSENSITIVE  = (1 << 4),
-#if 0
-	/* options specific to dates */
-	GNOME_PRINT_RANGE_DATE_TIME = (1 << 8),
-	GNOME_PRINT_RANGE_DATE_24HR = (1 << 9),
-	GNOME_PRINT_RANGE_DATE_MONDAY = (1 << 10),
-#endif
 };
 
-/* used to new call */
+/*
+ * Flags to specify, whether we want range and copies widgets
+ */
+
 enum GnomePrintDialogFlags {
 	GNOME_PRINT_DIALOG_RANGE = (1 << 0),
 	GNOME_PRINT_DIALOG_COPIES = (1 << 1)
@@ -73,6 +75,7 @@ enum GnomePrintDialogFlags {
  * The button numbers corresponding to the standard buttons
  * Used with the GnomeDialog "clicked" signal.
  */
+
 enum GnomePrintButtons {
 	GNOME_PRINT_DIALOG_RESPONSE_PRINT = 1,
 	GNOME_PRINT_DIALOG_RESPONSE_PREVIEW,
@@ -90,26 +93,34 @@ void gnome_print_dialog_construct_range_custom (GnomePrintDialog *dialog, GtkWid
 void gnome_print_dialog_construct_range_any (GnomePrintDialog *dialog, gint flags,
 					     GtkWidget *range_widget,
 					     const guchar *currentlabel, const guchar *rangelabel);
-#if 0
-void gnome_print_dialog_construct_range_date (GnomePrintDialog *gpd, gint flags,
-					      time_t start, time_t end,
-					      const guchar *currentlabel, const guchar *rangelabel);
-#endif
 void gnome_print_dialog_construct_range_page (GnomePrintDialog *gpd, gint flags,
 					      gint start, gint end,
 					      const guchar *currentlabel, const guchar *rangelabel);
 
 GnomePrintRangeType gnome_print_dialog_get_range (GnomePrintDialog *gpd);
 int gnome_print_dialog_get_range_page (GnomePrintDialog *gpd, gint *start, gint *end);
-#if 0
-int gnome_print_dialog_get_range_date (GnomePrintDialog *gpd, time_t *start, time_t *end);
-#endif
 
 void gnome_print_dialog_get_copies (GnomePrintDialog *gpd, gint *copies, gint *collate);
 void gnome_print_dialog_set_copies (GnomePrintDialog *gpd, gint copies, gint collate);
 
 GnomePrintConfig *gnome_print_dialog_get_config (GnomePrintDialog *gpd);
 
+/*
+ * I have disable date range, as it was not used, and it depended
+ * on libgnomeui widget, but we broke libgnomeprint(ui) free from
+ * libgnome(ui) completely.
+ *
+ * If somebody still needs those, it can be done by hand as custom range widget
+ */
+
+#if 0
+void gnome_print_dialog_construct_range_date (GnomePrintDialog *gpd, gint flags,
+					      time_t start, time_t end,
+					      const guchar *currentlabel, const guchar *rangelabel);
+#endif
+#if 0
+int gnome_print_dialog_get_range_date (GnomePrintDialog *gpd, time_t *start, time_t *end);
+#endif
 G_END_DECLS
 
 #endif /* ! _GNOME_PRINT_DIALOG */
