@@ -23,6 +23,8 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Cambridge, MA 02139, USA.
  */
 
+#include <config.h>
+
 #include <math.h>
 #include <libart_lgpl/art_misc.h>
 #include <libart_lgpl/art_vpath.h>
@@ -30,6 +32,8 @@
 #include <libart_lgpl/art_svp_vpath.h>
 #include <libart_lgpl/art_svp_wind.h>
 #include <libart_lgpl/art_rect_svp.h>
+#include <atk/atkobject.h>
+#include <atk/atkrelationset.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkvbox.h>
@@ -39,8 +43,13 @@
 #include <gtk/gtkoptionmenu.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
+#include <gtk/gtkspinbutton.h>
+#include <gtk/gtktogglebutton.h>
 #include <libgnomecanvas/gnome-canvas.h>
+#include <libgnomecanvas/gnome-canvas-util.h>
 #include <libgnomeprint/gnome-print-paper.h>
+#include <libgnomeprint/gnome-print-private.h>
+#include <libgnomeprint/gpa-private.h>
 #include "gnome-print-i18n.h"
 #include "gnome-print-paper-selector.h"
 
@@ -332,26 +341,6 @@ gnome_paper_preview_item_update (GnomeCanvasItem *item, gdouble *affine, ArtSVP
 	pp->LP2LY[5] = (t < 0) ? -t : 0.0;
 	PRINT_AFFINE ("LP2LY:", &pp->LP2LY[0]);
 
-#if 0
-	/* Find actual size */
-	/* Do not forget, that lw and lh are relative to pw and ph */
-	/* Also, do not forget layout affine rescaling in layout section */
-	if ((fabs (pp->lorient[1]) + fabs (pp->lorient[2])) > (fabs (pp->lorient[0]) + fabs (pp->lorient[3]))) {
-		pp->aw = pp->LYH;
-		pp->ah = pp->LYW;
-	} else {
-		pp->aw = pp->LYW;
-		pp->ah = pp->LYH;
-	}
-
-	/* Write correct translation to Logical orientation */
-	t = pp->aw * pp->lorient[0] + pp->ah * pp->lorient[2];
-	if (t < 0) pp->lorient[4] = -t;
-	t = pp->aw * pp->lorient[1] + pp->ah * pp->lorient[3];
-	if (t < 0) pp->lorient[5] = -t;
-	PRINT_AFFINE ("Logical Orientation:", &pp->lorient[0]);
-#endif
-
 	/* PPDC - Physical Page Dimensions on Canvas */
 
 	/* Now we calculate PPDP -> Canvas transformation */
@@ -899,7 +888,66 @@ gnome_paper_preview_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 GtkWidget *
 gnome_paper_preview_new (GnomePrintConfig *config)
 {
-	return gtk_type_new (GNOME_TYPE_PAPER_PREVIEW);
+	GnomePaperPreview *preview;
+	GnomePaperPreviewItem *item;
+	gdouble ml, mr, mt, mb, w, h;
+	GnomePrintLayoutData *lyd;
+
+	preview = GNOME_PAPER_PREVIEW (gtk_type_new (GNOME_TYPE_PAPER_PREVIEW));
+	item = GNOME_PAPER_PREVIEW_ITEM (preview->item);
+
+	/* Read out some important values */
+	w = 1;
+	gnome_print_config_get_length (config, GNOME_PRINT_KEY_PAPER_WIDTH, &w, NULL);
+	h = 1;
+	gnome_print_config_get_length (config, GNOME_PRINT_KEY_PAPER_HEIGHT, &h, NULL);
+	gnome_paper_preview_item_set_physical_size (item, w, h);
+
+	lyd = gnome_print_config_get_layout_data (config, NULL, NULL, NULL, NULL);
+	if (lyd != NULL) {
+		GSList *l = NULL;
+		gint i;
+
+		gnome_paper_preview_item_set_logical_orientation (item, lyd->lorient);
+		gnome_paper_preview_item_set_physical_orientation (item, lyd->porient);
+		for (i = lyd->num_pages; i ; ) {
+			l = g_slist_prepend (l, &lyd->pages[--i].matrix);
+		}
+		gnome_paper_preview_item_set_layout (item, l, lyd->lyw, lyd->lyh);
+		g_slist_free (l);
+		gnome_print_layout_data_free (lyd);
+	}
+	
+
+	ml = MM(10);
+	gnome_print_config_get_length (config, GNOME_PRINT_KEY_PAPER_MARGIN_LEFT, &ml, NULL);
+
+	mr = MM(10);
+	gnome_print_config_get_length (config, GNOME_PRINT_KEY_PAPER_MARGIN_RIGHT, &mr, NULL);
+
+	mt = MM(10);
+	gnome_print_config_get_length (config, GNOME_PRINT_KEY_PAPER_MARGIN_TOP, &mt, NULL);
+
+	mb = MM(10);
+	gnome_print_config_get_length (config, GNOME_PRINT_KEY_PAPER_MARGIN_BOTTOM, &mb, NULL);
+
+	gnome_paper_preview_item_set_physical_margins (item, ml, mr, mt, mb);
+
+	ml = MM(10);
+	gnome_print_config_get_length (config, GNOME_PRINT_KEY_PAGE_MARGIN_LEFT, &ml, NULL);
+
+	mr = MM(10);
+	gnome_print_config_get_length (config, GNOME_PRINT_KEY_PAGE_MARGIN_RIGHT, &mr, NULL);
+
+	mt = MM(10);
+	gnome_print_config_get_length (config, GNOME_PRINT_KEY_PAGE_MARGIN_TOP, &mt, NULL);
+
+	mb = MM(10);
+	gnome_print_config_get_length (config, GNOME_PRINT_KEY_PAGE_MARGIN_BOTTOM, &mb, NULL);
+
+	gnome_paper_preview_item_set_logical_margins (item, ml, mr, mt, mb);
+
+	return GTK_WIDGET(preview);
 }
 
 /*
@@ -909,9 +957,16 @@ gnome_paper_preview_new (GnomePrintConfig *config)
 struct _GnomePaperSelector {
 	GtkHBox box;
 
+	guint blocked : 1;
+
 	GnomePrintConfig *config;
 
 	GtkWidget *preview;
+
+	GtkWidget *pmenu, *pomenu, *lomenu, *lymenu;
+	GtkWidget *pw, *ph, *us;
+	GtkWidget *pf;
+	gdouble w, h;
 };
 
 struct _GnomePaperSelectorClass {
@@ -960,43 +1015,94 @@ gnome_paper_selector_class_init (GnomePaperSelectorClass *klass)
 static void
 gps_paper_activate (GtkWidget *widget, GnomePaperSelector *ps)
 {
-	GnomePrintPaper *paper;
+	GPANode *paper;
+	guchar *id;
+	const GnomePrintUnit *keyunit, *unit;
+	gdouble val;
 
-	paper = gtk_object_get_data (GTK_OBJECT (widget), "paper");
+	paper = gtk_object_get_data (GTK_OBJECT (widget), "node");
+	id = gpa_node_get_value (paper);
+	gnome_print_config_set (ps->config, GNOME_PRINT_KEY_PAPER_SIZE, id);
 
-	gnome_paper_preview_item_set_physical_size (GNOME_PAPER_PREVIEW_ITEM (GNOME_PAPER_PREVIEW (ps->preview)->item),
-						    paper->width, paper->height);
+	unit = gnome_print_unit_selector_get_unit (GNOME_PRINT_UNIT_SELECTOR (ps->us));
 
-	/* fixme: */
-	if (ps->config) {
-		gnome_print_config_set (ps->config, GNOME_PRINT_KEY_PAPER_SIZE, paper->name);
+	gnome_print_config_get_length (ps->config, GNOME_PRINT_KEY_PAPER_WIDTH, &val, &keyunit);
+	gnome_print_convert_distance (&val, keyunit, unit);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (ps->pw), val);
+	gnome_print_config_get_length (ps->config, GNOME_PRINT_KEY_PAPER_HEIGHT, &val, &keyunit);
+	gnome_print_convert_distance (&val, keyunit, unit);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (ps->ph), val);
+
+	/* fixme: Translation fucks that up */
+	if (!strcmp (id, "Custom")) {
+		gtk_widget_set_sensitive (ps->pw, TRUE);
+		gtk_widget_set_sensitive (ps->ph, TRUE);
+	} else {
+		gtk_widget_set_sensitive (ps->pw, FALSE);
+		gtk_widget_set_sensitive (ps->ph, FALSE);
 	}
+
+	g_free (id);
 }
 
-static void
-gps_paper_menu_create (GnomePaperSelector *selector, GtkWidget *optionmenu)
-{
-	GtkWidget *m, *i;
-	GList *papers, *l;
+/* Create menu */
+/* Lets be a bit more smart here (Lauris) */
 
-	papers = gnome_print_paper_get_list ();
+static void
+gps_menu_create (GtkWidget *om, GnomePrintConfig *config, const guchar *key, const guchar *emsg, GtkSignalFunc cb, gpointer cbdata)
+{
+	GPANode *cnode;
+	GtkWidget *m;
+	gint pos, sel;
 
 	m = gtk_menu_new ();
 	gtk_widget_show (m);
 
-	for (l = papers; l != NULL; l = l->next) {
-		GnomePrintPaper *paper;
-		paper = (GnomePrintPaper *) l->data;
-		i = gtk_menu_item_new_with_label (paper->name);
-		gtk_object_set_data (GTK_OBJECT (i), "paper", paper);
-		gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (gps_paper_activate), selector);
+	pos = sel = 0;
+	cnode = GNOME_PRINT_CONFIG_NODE (config);
+	if (cnode) {
+		GPANode *option;
+		guchar *optionkey, *current;
+		current = gnome_print_config_get (config, key);
+		optionkey = g_strdup_printf ("%s.Option", key);
+		option = gpa_node_get_path_node (cnode, optionkey);
+		g_free (optionkey);
+		if (option) {
+			GPANode *item;
+			for (item = gpa_node_get_child (option, NULL); item != NULL; item = gpa_node_get_child (option, item)) {
+				guchar *id, *name;
+				id = gpa_node_get_value (item);
+				name = gpa_node_get_path_value (item, "Name");
+				if (id && *id && name && *name) {
+					GtkWidget *i;
+					gpa_node_ref (item);
+					i = gtk_menu_item_new_with_label (name);
+					gtk_object_set_data_full (GTK_OBJECT (i), "node", item, (GtkDestroyNotify) gpa_node_unref);
+					gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (cb), cbdata);
+					gtk_widget_show (i);
+					gtk_menu_shell_append (GTK_MENU_SHELL (m), i);
+					if (current && !strcmp (id, current)) sel = pos;
+					pos += 1;
+				}
+				if (name) g_free (name);
+				if (id) g_free (id);
+				gpa_node_unref (item);
+			}
+			gpa_node_unref (option);
+		}
+		if (current) g_free (current);
+	}
+
+	if (pos < 1) {
+		GtkWidget *i;
+		i = gtk_menu_item_new_with_label (emsg);
 		gtk_widget_show (i);
 		gtk_menu_shell_append (GTK_MENU_SHELL (m), i);
 	}
 
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu), m);
-
-	gnome_print_paper_free_list (l);
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (om), m);
+	gtk_option_menu_set_history (GTK_OPTION_MENU (om), sel);
+	gtk_widget_set_sensitive (om, (pos > 0));
 }
 
 typedef struct {
@@ -1021,16 +1127,34 @@ static const GPPOrientation lorient[] = {
 static void
 gps_feed_orientation_activate (GtkWidget *widget, GnomePaperSelector *ps)
 {
+#if 0
 	GPPOrientation *o;
 
+#if 0
+	node = gtk_object_get_data (GTK_OBJECT (widget), "node");
+#endif
+
 	o = gtk_object_get_data (GTK_OBJECT (widget), "orientation");
 
 	gnome_paper_preview_item_set_physical_orientation (GNOME_PAPER_PREVIEW_ITEM (GNOME_PAPER_PREVIEW (ps->preview)->item), o->affine);
 
-	/* fixme: */
-	if (ps->config) {
-		gnome_print_config_set (ps->config, "Settings.Output.Media.PhysicalOrientation", o->id);
-	}
+	gnome_print_config_set (ps->config, GNOME_PRINT_KEY_PAPER_ORIENTATION, o->id);
+#else
+	GPANode *node;
+	guchar *id;
+	GnomePrintLayoutData *lyd;
+
+	node = gtk_object_get_data (GTK_OBJECT (widget), "node");
+	id = gpa_node_get_value (node);
+	gnome_print_config_set (ps->config, GNOME_PRINT_KEY_PAPER_ORIENTATION, id);
+	g_free (id);
+
+	/* Parse config */
+	lyd = gnome_print_config_get_layout_data (ps->config, NULL, NULL, NULL, NULL);
+	g_return_if_fail (lyd != NULL);
+	gnome_paper_preview_item_set_physical_orientation (GNOME_PAPER_PREVIEW_ITEM (GNOME_PAPER_PREVIEW (ps->preview)->item), lyd->porient);
+	gnome_print_layout_data_free (lyd);
+#endif
 }
 
 static void
@@ -1056,6 +1180,7 @@ gps_feed_orientation_menu_create (GnomePaperSelector *selector, GtkWidget *optio
 static void
 gps_page_orientation_activate (GtkWidget *widget, GnomePaperSelector *ps)
 {
+#if 0
 	GPPOrientation *o;
 
 	o = gtk_object_get_data (GTK_OBJECT (widget), "orientation");
@@ -1066,6 +1191,22 @@ gps_page_orientation_activate (GtkWidget *widget, GnomePaperSelector *ps)
 	if (ps->config) {
 		gnome_print_config_set (ps->config, "Settings.Document.Page.LogicalOrientation", o->id);
 	}
+#else
+	GPANode *node;
+	guchar *id;
+	GnomePrintLayoutData *lyd;
+
+	node = gtk_object_get_data (GTK_OBJECT (widget), "node");
+	id = gpa_node_get_value (node);
+	gnome_print_config_set (ps->config, GNOME_PRINT_KEY_PAGE_ORIENTATION, id);
+	g_free (id);
+
+	/* Parse config */
+	lyd = gnome_print_config_get_layout_data (ps->config, NULL, NULL, NULL, NULL);
+	g_return_if_fail (lyd != NULL);
+	gnome_paper_preview_item_set_logical_orientation (GNOME_PAPER_PREVIEW_ITEM (GNOME_PAPER_PREVIEW (ps->preview)->item), lyd->lorient);
+	gnome_print_layout_data_free (lyd);
+#endif
 }
 
 static void
@@ -1120,6 +1261,7 @@ static const GPPLayout layout[] = {
 static void
 gps_layout_activate (GtkWidget *widget, GnomePaperSelector *ps)
 {
+#if 0
 	GPPLayout *ly;
 	GSList *l;
 	gint i;
@@ -1138,6 +1280,30 @@ gps_layout_activate (GtkWidget *widget, GnomePaperSelector *ps)
 	if (ps->config) {
 		gnome_print_config_set (ps->config, "Settings.Document.Page.Layout", ly->id);
 	}
+#else
+	GPANode *node;
+	guchar *id;
+	GnomePrintLayoutData *lyd;
+	GSList *l;
+	gint i;
+
+	node = gtk_object_get_data (GTK_OBJECT (widget), "node");
+	id = gpa_node_get_value (node);
+	gnome_print_config_set (ps->config, GNOME_PRINT_KEY_LAYOUT, id);
+	g_free (id);
+
+	/* Parse config */
+	lyd = gnome_print_config_get_layout_data (ps->config, NULL, NULL, NULL, NULL);
+	g_return_if_fail (lyd != NULL);
+	l = NULL;
+	for (i = 0; i < lyd->num_pages; i++) {
+		l = g_slist_prepend (l, &lyd->pages[i].matrix);
+	}
+	l = g_slist_reverse (l);
+	gnome_paper_preview_item_set_layout (GNOME_PAPER_PREVIEW_ITEM (GNOME_PAPER_PREVIEW (ps->preview)->item), l, lyd->lyw, lyd->lyh);
+	g_slist_free (l);
+	gnome_print_layout_data_free (lyd);
+#endif
 }
 
 static void
@@ -1161,77 +1327,75 @@ gps_layout_menu_create (GnomePaperSelector *selector, GtkWidget *optionmenu)
 }
 
 static void
-gnome_paper_selector_init (GnomePaperSelector *selector)
+gps_psize_value_changed (GtkAdjustment *adj, GnomePaperSelector *ps)
 {
-	GtkWidget *vb, *f, *t, *l, *m;
+	const GnomePrintUnit *unit;
+	gdouble w, h;
 
-	selector->config = NULL;
+	unit = gnome_print_unit_selector_get_unit (GNOME_PRINT_UNIT_SELECTOR (ps->us));
+	w = gtk_spin_button_get_value (GTK_SPIN_BUTTON (ps->pw));
+#ifdef GPP_VERBOSE
+	g_print ("Width %g\n", w);
+#endif	
 
-	/* VBox for controls */
-	vb = gtk_vbox_new (FALSE, 4);
-	gtk_widget_show (vb);
-	gtk_box_pack_start (GTK_BOX (selector), vb, FALSE, FALSE, 4);
+	gnome_print_convert_distance (&w, unit, GNOME_PRINT_PS_UNIT);
+	h = gtk_spin_button_get_value (GTK_SPIN_BUTTON (ps->ph));
+#ifdef GPP_VERBOSE
+	g_print ("Height %g\n", h);
+#endif
+	
+	gnome_print_convert_distance (&h, unit, GNOME_PRINT_PS_UNIT);
+#ifdef GPP_VERBOSE
+	g_print ("New values %g %g\n", w, h);
+#endif
 
-	/* Create frame for selection menus */
-	f = gtk_frame_new (_("Paper and layout"));
-	gtk_widget_show (f);
-	gtk_box_pack_start (GTK_BOX (vb), f, FALSE, FALSE, 0);
+	if ((fabs (ps->w - w) < 0.1) && (fabs (ps->h - h) < 0.1)) return;
+#ifdef GPP_VERBOSE
+	g_print ("Setting preview %g %g\n", w, h);
+#endif
+	gnome_paper_preview_item_set_physical_size (GNOME_PAPER_PREVIEW_ITEM (GNOME_PAPER_PREVIEW (ps->preview)->item), w, h);
+	ps->w = w;
+	ps->h = h;
+}
 
-	/* Create table for packing menus */
-	t = gtk_table_new (2, 4, FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (t), 4);
-	gtk_widget_show (t);
-	gtk_container_add (GTK_CONTAINER (f), t);
+static void
+gps_preview_toggled (GtkToggleButton *tb, GnomePaperSelector *ps)
+{
+	if (gtk_toggle_button_get_active (tb)) {
+#ifdef GPP_VERBOSE
+		g_print ("show\n");
+#endif
+		gtk_widget_show (ps->pf);
+	} else {
+#ifdef GPP_VERBOSE
+		g_print ("hide\n");
+#endif
+		gtk_widget_hide (ps->pf);
+	}
+}
 
-	l = gtk_label_new (_("Paper size"));
-	gtk_widget_show (l);
-	gtk_table_attach_defaults (GTK_TABLE (t), l, 0, 1, 0, 1);
+static void
+gps_set_labelled_by_relation (GtkWidget *widget, GtkWidget *label)
+{
+	AtkRelationSet *relation_set;
+	AtkRelation *relation;
+	AtkObject *relation_targets[1];
+	AtkObject *atko;
 
-	m = gtk_option_menu_new ();
-	gtk_widget_show (m);
-	gtk_table_attach_defaults (GTK_TABLE (t), m, 1, 2, 0, 1);
+	atko = gtk_widget_get_accessible (widget);
+	relation_set = atk_object_ref_relation_set (atko);
+	relation_targets[0] = gtk_widget_get_accessible (label);
+	relation = atk_relation_new (relation_targets, 1,
+				     ATK_RELATION_LABELLED_BY);
+	atk_relation_set_add (relation_set, relation);
+	g_object_unref (G_OBJECT (relation));
+	g_object_unref (G_OBJECT (relation_set));
+}
 
-	gps_paper_menu_create (selector, m);
-
-	l = gtk_label_new (_("Feed orientation"));
-	gtk_widget_show (l);
-	gtk_table_attach_defaults (GTK_TABLE (t), l, 0, 1, 1, 2);
-
-	m = gtk_option_menu_new ();
-	gtk_widget_show (m);
-	gtk_table_attach_defaults (GTK_TABLE (t), m, 1, 2, 1, 2);
-
-	gps_feed_orientation_menu_create (selector, m);
-
-	l = gtk_label_new (_("Page orientation"));
-	gtk_widget_show (l);
-	gtk_table_attach_defaults (GTK_TABLE (t), l, 0, 1, 2, 3);
-
-	m = gtk_option_menu_new ();
-	gtk_widget_show (m);
-	gtk_table_attach_defaults (GTK_TABLE (t), m, 1, 2, 2, 3);
-
-	gps_page_orientation_menu_create (selector, m);
-
-	l = gtk_label_new (_("Layout"));
-	gtk_widget_show (l);
-	gtk_table_attach_defaults (GTK_TABLE (t), l, 0, 1, 3, 4);
-
-	m = gtk_option_menu_new ();
-	gtk_widget_show (m);
-	gtk_table_attach_defaults (GTK_TABLE (t), m, 1, 2, 3, 4);
-
-	gps_layout_menu_create (selector, m);
-
-	/* Preview frame */
-	f = gtk_frame_new (_("Preview"));
-	gtk_widget_show (f);
-	gtk_box_pack_start (GTK_BOX (selector), f, TRUE, TRUE, 4);
-
-	selector->preview = gnome_paper_preview_new (NULL);
-	gtk_widget_set_usize (selector->preview, 320, 320);
-	gtk_widget_show (selector->preview);
-	gtk_container_add (GTK_CONTAINER (f), selector->preview);
+static void
+gnome_paper_selector_init (GnomePaperSelector *ps)
+{
+	ps->config = NULL;
 }
 
 static void
@@ -1250,6 +1414,176 @@ gnome_paper_selector_finalize (GObject *object)
 	G_OBJECT_CLASS (item_parent_class)->finalize (object);
 }
 
+#define PAD 6
+static void
+gnome_paper_selector_construct (GnomePaperSelector *ps)
+{
+	GtkWidget *vb, *f, *t, *l;
+#if 0
+	GtkWidget *s, *hb, *b;
+#endif
+	GtkObject *wa, *ha;
+	AtkObject *atko;
+	gdouble config_height, config_width;
+	guchar *id;
+
+	g_return_if_fail (ps != NULL);
+	g_return_if_fail (ps->config != NULL);
+
+	gtk_box_set_spacing (GTK_BOX (ps), PAD);
+
+	/* VBox for controls */
+	vb = gtk_vbox_new (FALSE, PAD);
+	gtk_widget_show (vb);
+	gtk_box_pack_start (GTK_BOX (ps), vb, FALSE, FALSE, 0);
+
+	/* Create frame for selection menus */
+	f = gtk_frame_new (_("Paper and layout"));
+	gtk_widget_show (f);
+	gtk_box_pack_start (GTK_BOX (vb), f, FALSE, FALSE, 0);
+
+	/* Create table for packing menus */
+	t = gtk_table_new (4, 6, FALSE);
+	gtk_widget_show (t);
+	gtk_container_set_border_width (GTK_CONTAINER (t), PAD);
+	gtk_table_set_row_spacings (GTK_TABLE (t), 2);
+	gtk_table_set_col_spacings (GTK_TABLE (t), 4);
+	gtk_container_add (GTK_CONTAINER (f), t);
+
+	l = gtk_label_new_with_mnemonic (_("Paper _size:"));
+	gtk_widget_show (l);
+	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
+	gtk_table_attach_defaults (GTK_TABLE (t), l, 0, 1, 0, 1);
+
+	ps->pmenu = gtk_option_menu_new ();
+	gtk_widget_show (ps->pmenu);
+	gtk_table_attach_defaults (GTK_TABLE (t), ps->pmenu, 1, 4, 0, 1);
+	gtk_label_set_mnemonic_widget ((GtkLabel *) l, ps->pmenu);
+	gps_set_labelled_by_relation (ps->pmenu, l);
+
+	/* Create spinbuttons for paper size */
+	l = gtk_label_new_with_mnemonic (_("_Width:"));
+	gtk_widget_show (l);
+	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
+	gtk_table_attach_defaults (GTK_TABLE (t), l, 1, 2, 1, 2);
+
+	config_width = 1;
+	gnome_print_config_get_length (ps->config, GNOME_PRINT_KEY_PAPER_WIDTH, 
+				       &config_width, NULL);
+	ps->w = config_width;
+	wa = gtk_adjustment_new (config_width, 0.0001, 10000, 1, 10, 10);
+	ps->pw = gtk_spin_button_new (GTK_ADJUSTMENT (wa), 1, 2);
+	gtk_widget_show (ps->pw);
+	gtk_table_attach_defaults (GTK_TABLE (t), ps->pw, 2, 3, 1, 2);
+	gtk_label_set_mnemonic_widget ((GtkLabel *) l, ps->pw);
+	gps_set_labelled_by_relation (ps->pw, l);
+
+	l = gtk_label_new_with_mnemonic (_("_Height:"));
+	gtk_widget_show (l);
+	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
+	gtk_table_attach_defaults (GTK_TABLE (t), l, 1, 2, 2, 3);
+
+	config_height = 1;
+	gnome_print_config_get_length (ps->config, GNOME_PRINT_KEY_PAPER_HEIGHT, 
+				       &config_height, NULL);
+	ps->h = config_height;
+	ha = gtk_adjustment_new (config_height, 0.0001, 10000, 1, 10, 10);
+	ps->ph = gtk_spin_button_new (GTK_ADJUSTMENT (ha), 1, 2);
+	gtk_widget_show (ps->ph);
+	gtk_table_attach_defaults (GTK_TABLE (t), ps->ph, 2, 3, 2, 3);
+	gtk_label_set_mnemonic_widget ((GtkLabel *) l, ps->ph);
+	gps_set_labelled_by_relation (ps->ph, l);
+
+	ps->us = gnome_print_unit_selector_new (GNOME_PRINT_UNIT_ABSOLUTE);
+	gtk_widget_show (ps->us);
+	gtk_table_attach_defaults (GTK_TABLE (t), ps->us, 3, 4, 1, 2);
+	gnome_print_unit_selector_add_adjustment (GNOME_PRINT_UNIT_SELECTOR (ps->us), GTK_ADJUSTMENT (wa));
+	gnome_print_unit_selector_add_adjustment (GNOME_PRINT_UNIT_SELECTOR (ps->us), GTK_ADJUSTMENT (ha));
+	gtk_signal_connect (GTK_OBJECT (wa), "value_changed", GTK_SIGNAL_FUNC (gps_psize_value_changed), ps);
+	gtk_signal_connect (GTK_OBJECT (ha), "value_changed", GTK_SIGNAL_FUNC (gps_psize_value_changed), ps);
+	atko = gtk_widget_get_accessible (ps->us);
+	atk_object_set_name (atko, _("Metric selector"));
+	atk_object_set_description (atko, _("Specifies the metric to use when setting the width and height of the paper"));
+
+	/* Feed orientation */
+	l = gtk_label_new_with_mnemonic (_("_Feed orientation:"));
+	gtk_widget_show (l);
+	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
+	gtk_table_attach_defaults (GTK_TABLE (t), l, 0, 1, 3, 4);
+
+	ps->pomenu = gtk_option_menu_new ();
+	gtk_widget_show (ps->pomenu);
+	gtk_table_attach_defaults (GTK_TABLE (t), ps->pomenu, 1, 4, 3, 4);
+	gtk_label_set_mnemonic_widget ((GtkLabel *) l, ps->pomenu);
+	gps_set_labelled_by_relation (ps->pomenu, l);
+
+	l = gtk_label_new_with_mnemonic (_("Page _orientation:"));
+	gtk_widget_show (l);
+	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
+	gtk_table_attach_defaults (GTK_TABLE (t), l, 0, 1, 4, 5);
+
+	ps->lomenu = gtk_option_menu_new ();
+	gtk_widget_show (ps->lomenu);
+	gtk_table_attach_defaults (GTK_TABLE (t), ps->lomenu, 1, 4, 4, 5);
+	gtk_label_set_mnemonic_widget ((GtkLabel *) l, ps->lomenu);
+	gps_set_labelled_by_relation (ps->lomenu, l);
+
+	l = gtk_label_new_with_mnemonic (_("_Layout:"));
+	gtk_widget_show (l);
+	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
+	gtk_table_attach_defaults (GTK_TABLE (t), l, 0, 1, 5, 6);
+
+	ps->lymenu = gtk_option_menu_new ();
+	gtk_widget_show (ps->lymenu);
+	gtk_table_attach_defaults (GTK_TABLE (t), ps->lymenu, 1, 4, 5, 6);
+	gtk_label_set_mnemonic_widget ((GtkLabel *) l, ps->lymenu);
+	gps_set_labelled_by_relation (ps->lymenu, l);
+
+#if 0
+	/* Show hide button */
+	hb = gtk_hbox_new (FALSE, PAD);
+	gtk_widget_show (hb);
+	gtk_box_pack_end (GTK_BOX (vb), hb, FALSE, FALSE, 0);
+	b = gtk_toggle_button_new_with_label (_("Toggle preview"));
+	gtk_signal_connect (GTK_OBJECT (b), "toggled", GTK_SIGNAL_FUNC (gps_preview_toggled), ps);
+	gtk_widget_show (b);
+	gtk_box_pack_end (GTK_BOX (hb), b, FALSE, FALSE, 0);
+#endif
+	/* Preview frame */
+	ps->pf = gtk_frame_new (_("Preview"));
+	gtk_widget_show (ps->pf);
+	gtk_box_pack_start (GTK_BOX (ps), ps->pf, TRUE, TRUE, 0);
+
+	ps->preview = gnome_paper_preview_new (ps->config);
+	gtk_widget_set_usize (ps->preview, 160, 160);
+	gtk_widget_show (ps->preview);
+	gtk_container_add (GTK_CONTAINER (ps->pf), ps->preview);
+
+	atko = gtk_widget_get_accessible (ps->preview);
+	atk_object_set_name (atko, _("Preview"));
+	atk_object_set_description (atko, _("Preview of the page size, orientation and layout"));
+
+	/* Fill optionmenus from config */
+	gps_menu_create (ps->pmenu, ps->config, GNOME_PRINT_KEY_PAPER_SIZE, _("No papers defined"),
+			 GTK_SIGNAL_FUNC (gps_paper_activate), ps);
+	id = gnome_print_config_get (ps->config, GNOME_PRINT_KEY_PAPER_SIZE);
+	if (!id || !strcmp (id, "Custom")) {
+		gtk_widget_set_sensitive (ps->pw, TRUE);
+		gtk_widget_set_sensitive (ps->ph, TRUE);
+	} else {
+		gtk_widget_set_sensitive (ps->pw, FALSE);
+		gtk_widget_set_sensitive (ps->ph, FALSE);
+	}
+	g_free (id);
+	gps_menu_create (ps->pomenu, ps->config, GNOME_PRINT_KEY_PAPER_ORIENTATION, _("No orientations defined"),
+			 GTK_SIGNAL_FUNC (gps_feed_orientation_activate), ps);
+	gps_menu_create (ps->lomenu, ps->config, GNOME_PRINT_KEY_PAGE_ORIENTATION, _("No orientations defined"),
+			 GTK_SIGNAL_FUNC (gps_page_orientation_activate), ps);
+	gps_menu_create (ps->lymenu, ps->config, GNOME_PRINT_KEY_LAYOUT, _("No layouts defined"),
+			 GTK_SIGNAL_FUNC (gps_layout_activate), ps);
+
+}
+
 GtkWidget *
 gnome_paper_selector_new (GnomePrintConfig *config)
 {
@@ -1260,11 +1594,282 @@ gnome_paper_selector_new (GnomePrintConfig *config)
 	/* fixme: */
 	if (config) {
 		selector->config = gnome_print_config_ref (config);
+	} else {
+		selector->config = gnome_print_config_default ();
 	}
 
+	gnome_paper_selector_construct (selector);
+
+
 	return (GtkWidget *) selector;
 }
 
+/*
+ * GnomePrintUnitSelector widget
+ */
+
+struct _GnomePrintUnitSelector {
+	GtkHBox box;
+
+	GtkWidget *menu;
+
+	guint bases;
+	GList *units;
+	const GnomePrintUnit *unit;
+	gdouble ctmscale, devicescale;
+	guint plural : 1;
+	guint abbr : 1;
+
+	GList *adjustments;
+};
+
+struct _GnomePrintUnitSelectorClass {
+	GtkHBoxClass parent_class;
+};
+
+static void gnome_print_unit_selector_class_init (GnomePrintUnitSelectorClass *klass);
+static void gnome_print_unit_selector_init (GnomePrintUnitSelector *selector);
+static void gnome_print_unit_selector_finalize (GObject *object);
+
+static GtkHBoxClass *unit_selector_parent_class;
+
+GType
+gnome_print_unit_selector_get_type (void)
+{
+	static GType type = 0;
+	if (!type) {
+		static const GTypeInfo info = {
+			sizeof (GnomePrintUnitSelectorClass),
+			NULL, NULL,
+			(GClassInitFunc) gnome_print_unit_selector_class_init,
+			NULL, NULL,
+			sizeof (GnomePrintUnitSelector),
+			0,
+			(GInstanceInitFunc) gnome_print_unit_selector_init
+		};
+		type = g_type_register_static (GTK_TYPE_HBOX, "GnomePrintUnitSelector", &info, 0);
+	}
+	return type;
+}
+
+static void
+gnome_print_unit_selector_class_init (GnomePrintUnitSelectorClass *klass)
+{
+	GObjectClass *object_class;
+	GtkWidgetClass *widget_class;
+
+	object_class = G_OBJECT_CLASS (klass);
+	widget_class = GTK_WIDGET_CLASS (klass);
+
+	unit_selector_parent_class = g_type_class_peek_parent (klass);
+
+	object_class->finalize = gnome_print_unit_selector_finalize;
+}
+
+static void
+gnome_print_unit_selector_init (GnomePrintUnitSelector *us)
+{
+	us->ctmscale = 1.0;
+	us->devicescale = 1.0;
+	us->abbr = FALSE;
+	us->plural = TRUE;
+
+	us->menu = gtk_option_menu_new ();
+	gtk_widget_show (us->menu);
+	gtk_box_pack_start (GTK_BOX (us), us->menu, TRUE, TRUE, 0);
+}
+
+static void
+gnome_print_unit_selector_finalize (GObject *object)
+{
+	GnomePrintUnitSelector *selector;
+
+	selector = GNOME_PRINT_UNIT_SELECTOR (object);
+
+	if (selector->menu) {
+		selector->menu = NULL;
+	}
+
+	while (selector->adjustments) {
+		g_object_unref (G_OBJECT (selector->adjustments->data));
+		selector->adjustments = g_list_remove (selector->adjustments, selector->adjustments->data);
+	}
+
+	if (selector->units) {
+		gnome_print_unit_free_list (selector->units);
+	}
+
+	selector->unit = NULL;
+
+	G_OBJECT_CLASS (item_parent_class)->finalize (object);
+}
+
+GtkWidget *
+gnome_print_unit_selector_new (guint bases)
+{
+	GnomePrintUnitSelector *us;
+
+	us = gtk_type_new (GNOME_TYPE_PRINT_UNIT_SELECTOR);
+
+	gnome_print_unit_selector_set_bases (us, bases);
+
+	return (GtkWidget *) us;
+}
+
+const GnomePrintUnit *
+gnome_print_unit_selector_get_unit (GnomePrintUnitSelector *us)
+{
+	g_return_val_if_fail (us != NULL, NULL);
+	g_return_val_if_fail (GNOME_IS_PRINT_UNIT_SELECTOR (us), NULL);
+
+	return us->unit;
+}
+
+static void
+gpus_unit_activate (GtkWidget *widget, GnomePrintUnitSelector *us)
+{
+	const GnomePrintUnit *unit, *old;
+	GList *l;
+
+	unit = g_object_get_data (G_OBJECT (widget), "unit");
+	g_return_if_fail (unit != NULL);
+
+#ifdef GPP_VERBOSE
+	g_print ("Old unit %s new unit %s\n", us->unit->name, unit->name);
+#endif
+
+	old = us->unit;
+	us->unit = unit;
+
+	/* Recalculate adjustments */
+	for (l = us->adjustments; l != NULL; l = l->next) {
+		GtkAdjustment *adj;
+		gdouble val;
+		adj = GTK_ADJUSTMENT (l->data);
+		val = gtk_adjustment_get_value (adj);
+#ifdef GPP_VERBOSE
+		g_print ("Old val %g ... ", val);
+#endif	
+		gnome_print_convert_distance_full (&val, old, unit, us->ctmscale, us->devicescale);
+#ifdef GPP_VERBOSE
+		g_print ("new val %g\n", val);
+#endif
+		gtk_adjustment_set_value (adj, val);
+	}
+}
+
+static void
+gpus_rebuild_menu (GnomePrintUnitSelector *us)
+{
+	GtkWidget *m, *i;
+	GList *l;
+	gint pos, p;
+
+	if (GTK_OPTION_MENU (us->menu)->menu) {
+		gtk_option_menu_remove_menu (GTK_OPTION_MENU (us->menu));
+	}
+
+	m = gtk_menu_new ();
+	gtk_widget_show (m);
+
+	pos = p = 0;
+	for (l = us->units; l != NULL; l = l->next) {
+		const GnomePrintUnit *u;
+		u = l->data;
+		i = gtk_menu_item_new_with_label ((us->abbr) ? (us->plural) ? u->abbr_plural : u->abbr : (us->plural) ? u->plural : u->name);
+		g_object_set_data (G_OBJECT (i), "unit", (gpointer) u);
+		gtk_signal_connect (GTK_OBJECT (i), "activate", GTK_SIGNAL_FUNC (gpus_unit_activate), us);
+		gtk_widget_show (i);
+		gtk_menu_shell_append (GTK_MENU_SHELL (m), i);
+		if (u == us->unit) pos = p;
+		p += 1;
+	}
+
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (us->menu), m);
+
+	gtk_option_menu_set_history (GTK_OPTION_MENU (us->menu), pos);
+}
+
+void
+gnome_print_unit_selector_set_bases (GnomePrintUnitSelector *us, guint bases)
+{
+	GList *units;
+
+	g_return_if_fail (us != NULL);
+	g_return_if_fail (GNOME_IS_PRINT_UNIT_SELECTOR (us));
+
+	if (bases == us->bases) return;
+
+	units = gnome_print_unit_get_list (bases);
+	g_return_if_fail (units != NULL);
+	gnome_print_unit_free_list (us->units);
+	us->units = units;
+	us->unit = units->data;
+
+	gpus_rebuild_menu (us);
+}
+
+void
+gnome_print_unit_selector_set_unit (GnomePrintUnitSelector *us, const GnomePrintUnit *unit)
+{
+	const GnomePrintUnit *old;
+	GList *l;
+	gint pos;
+
+	g_return_if_fail (us != NULL);
+	g_return_if_fail (GNOME_IS_PRINT_UNIT_SELECTOR (us));
+	g_return_if_fail (unit != NULL);
+
+	if (unit == us->unit) return;
+
+	pos = g_list_index (us->units, unit);
+	g_return_if_fail (pos >= 0);
+
+	gtk_option_menu_set_history (GTK_OPTION_MENU (us->menu), pos);
+
+	old = us->unit;
+	us->unit = unit;
+
+	/* Recalculate adjustments */
+	for (l = us->adjustments; l != NULL; l = l->next) {
+		GtkAdjustment *adj;
+		gdouble val;
+		adj = GTK_ADJUSTMENT (l->data);
+		val = gtk_adjustment_get_value (adj);
+		gnome_print_convert_distance_full (&val, old, unit, us->ctmscale, us->devicescale);
+		gtk_adjustment_set_value (adj, val);
+	}
+}
+
+void
+gnome_print_unit_selector_add_adjustment (GnomePrintUnitSelector *us, GtkAdjustment *adj)
+{
+	g_return_if_fail (us != NULL);
+	g_return_if_fail (GNOME_IS_PRINT_UNIT_SELECTOR (us));
+	g_return_if_fail (adj != NULL);
+	g_return_if_fail (GTK_IS_ADJUSTMENT (adj));
+
+	g_return_if_fail (!g_list_find (us->adjustments, adj));
+
+	g_object_ref (G_OBJECT (adj));
+	us->adjustments = g_list_prepend (us->adjustments, adj);
+}
+
+void
+gnome_print_unit_selector_remove_adjustment (GnomePrintUnitSelector *us, GtkAdjustment *adj)
+{
+	g_return_if_fail (us != NULL);
+	g_return_if_fail (GNOME_IS_PRINT_UNIT_SELECTOR (us));
+	g_return_if_fail (adj != NULL);
+	g_return_if_fail (GTK_IS_ADJUSTMENT (adj));
+
+	g_return_if_fail (g_list_find (us->adjustments, adj));
+
+	us->adjustments = g_list_remove (us->adjustments, adj);
+	g_object_unref (G_OBJECT (adj));
+}
+
+
 #if 0
 
 /* #define PAGELAYOUT_TEST */
@@ -1274,9 +1879,6 @@ gnome_paper_selector_new (GnomePrintConfig *config)
 
 #include <string.h>
 
-#include <libgnome/gnome-i18n.h>
-#include <libgnomeui/gnome-uidefs.h>
-
 #include "gnome-paper-selector.h"
 #include "gnome-unit-spinner.h"
 
@@ -2260,47 +2862,6 @@ gnome_paper_selector_get_bottom_margin(GnomePaperSelector *gspaper);
 }
 #endif
 
-
-#ifdef PAGELAYOUT_TEST
-
-void
-changed_signal(GnomePaperSelector *self)
-{
-  g_message("changed");
-}
-void
-fittopage_signal(GnomePaperSelector *self)
-{
-  g_message("fit to page");
-}
-
-void
-main(int argc, char **argv)
-{
-  GtkWidget *win, *ps;
-
-  gtk_init(&argc, &argv);
-
-  win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_title(GTK_WINDOW(win), _("Page Setup"));
-  gtk_signal_connect(GTK_OBJECT(win), "destroy",
-		     GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
-
-  ps = gnome_paper_selector_new();
-  gtk_container_set_border_width(GTK_CONTAINER(ps), 5);
-  gtk_container_add(GTK_CONTAINER(win), ps);
-  gtk_widget_show(ps);
-
-  gtk_signal_connect(GTK_OBJECT(ps), "changed",
-		     GTK_SIGNAL_FUNC(changed_signal), NULL);
-  gtk_signal_connect(GTK_OBJECT(ps), "fittopage",
-		     GTK_SIGNAL_FUNC(fittopage_signal), NULL);
-
-  gtk_widget_show(win);
-  gtk_main();
-}
-
-#endif
-
 #endif /* The whole thing */
 
+
