@@ -12,12 +12,14 @@
  *
  */
 
+#include <config.h>
+
 #include <string.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
 #include <gtk/gtkoptionmenu.h>
-#include <libgnome/gnome-i18n.h>
+#include "../gnome-print-i18n.h"
 #include "printer-selector.h"
 
 static void gpa_printer_selector_class_init (GPAPrinterSelectorClass *klass);
@@ -109,33 +111,16 @@ gpa_printer_selector_construct (GPAWidget *widget)
 	GPANode *node;
 
 	selector = GPA_PRINTER_SELECTOR (widget);
-	node = widget->node;
+	node = GNOME_PRINT_CONFIG_NODE (widget->config);
 
 	selector->printers = gpa_node_get_path_node (node, "Globals.Printers");
 	g_return_val_if_fail (selector->printers != NULL, FALSE);
 
-	gtk_signal_connect_while_alive (GTK_OBJECT (selector->printers), "modified",
-					GTK_SIGNAL_FUNC (gpa_ps_printers_modified), selector, GTK_OBJECT (selector));
-	gtk_signal_connect_while_alive (GTK_OBJECT (node), "modified",
-					GTK_SIGNAL_FUNC (gpa_ps_config_modified), selector, GTK_OBJECT (selector));
-
 	gpa_ps_rebuild_menu (selector);
 
 	return TRUE;
 }
 
-static void
-gpa_ps_printers_modified (GPANode *printers, GPAPrinterSelector *ps)
-{
-	g_print ("Printers modified\n");
-	gpa_ps_rebuild_menu (ps);
-}
-
-static void
-gpa_ps_config_modified (GPANode *config, GPAPrinterSelector *ps)
-{
-}
-
 static void
 gpa_ps_rebuild_menu (GPAPrinterSelector *ps)
 {
@@ -203,19 +188,16 @@ gpa_ps_rebuild_menu (GPAPrinterSelector *ps)
 		item = gtk_menu_item_new ();
 		gtk_widget_show (item);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-#if 0
 		item = gtk_menu_item_new_with_label (_("Manage printers"));
 		gtk_signal_connect (GTK_OBJECT (item), "activate", GTK_SIGNAL_FUNC (gpa_ps_manage_printers_activate), ps);
-#else
-		item = gtk_menu_item_new_with_label (_("Add new printer"));
-		gtk_signal_connect (GTK_OBJECT (item), "activate", GTK_SIGNAL_FUNC (gpa_ps_add_printer_activate), ps);
-#endif
+		gtk_widget_set_sensitive (item, FALSE);
 		gtk_widget_show (item);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	} else {
 		/* No printers */
 		item = gtk_menu_item_new_with_label (_("Add new printer"));
 		gtk_signal_connect (GTK_OBJECT (item), "activate", GTK_SIGNAL_FUNC (gpa_ps_add_printer_activate), ps);
+		gtk_widget_set_sensitive (item, FALSE);
 		gtk_widget_show (item);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	}
@@ -244,7 +226,7 @@ gpa_ps_menuitem_activate (GtkWidget *widget, gint index)
 	value = gpa_node_get_value (printer);
 	g_return_if_fail (value != NULL);
 
-	gpa_node_set_path_value (gpaw->node, "Printer", value);
+	gnome_print_config_set (gpaw->config, "Printer", value);
 
 	g_free (value);
 }
@@ -252,18 +234,12 @@ gpa_ps_menuitem_activate (GtkWidget *widget, gint index)
 static void
 gpa_ps_add_printer_activate (GtkWidget *widget, GPAWidget *gpaw)
 {
-#if 0
-	GtkWidget *apd;
-
-	apd = gpa_add_printer_dialog_new (gpaw->node);
-
-	gtk_widget_show (apd);
-#endif
+	g_print ("Add printer activated\n");
 }
 
 static void
 gpa_ps_manage_printers_activate (GtkWidget *widget, GPAWidget *gpaw)
 {
-	g_print ("Manage printers\n");
+	g_print ("Manage printers activated\n");
 }
 
